var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { ComputeBudgetProgram, } from "@solana/web3.js";
// Add priority fee according to 50th percentile of recent fees paid
const DEFAULT_FEE_PERCENTILE = 0.5;
export function addComputeBudget(connection, transaction, lockedWritableAccounts = [], feePercentile = DEFAULT_FEE_PERCENTILE, minPriorityFee = 0, throwOnSimulateError = false) {
    return __awaiter(this, void 0, void 0, function* () {
        if (lockedWritableAccounts.length === 0) {
            lockedWritableAccounts = transaction.instructions
                .flatMap((ix) => ix.keys)
                .map((k) => (k.isWritable ? k.pubkey : null))
                .filter((k) => k !== null);
        }
        const ixs = yield determineComputeBudget(connection, transaction, lockedWritableAccounts, feePercentile, minPriorityFee, throwOnSimulateError);
        transaction.add(...ixs);
    });
}
export function addComputeBudgetStatic(transaction, lockedWritableAccounts = []) {
    return __awaiter(this, void 0, void 0, function* () {
        if (lockedWritableAccounts.length === 0) {
            lockedWritableAccounts = transaction.instructions
                .flatMap((ix) => ix.keys)
                .map((k) => (k.isWritable ? k.pubkey : null))
                .filter((k) => k !== null);
        }
        const ixs = yield determineComputeBudgetStatic();
        transaction.add(...ixs);
    });
}
export function determineComputeBudgetStatic() {
    return __awaiter(this, void 0, void 0, function* () {
        return [
            ComputeBudgetProgram.setComputeUnitLimit({
                units: Number(500000),
            }),
            ComputeBudgetProgram.setComputeUnitPrice({
                microLamports: BigInt(100000),
            }),
        ];
    });
}
export function determineComputeBudget(connection, transaction, lockedWritableAccounts = [], feePercentile = DEFAULT_FEE_PERCENTILE, minPriorityFee = 0, throwOnSimulateError = false) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        let computeBudget = 250000;
        let priorityFee = 1;
        try {
            // TODO: Use non-deprecated method and pass a commitment level
            const simulateResponse = yield connection.simulateTransaction(transaction);
            if (simulateResponse.value.err) {
                console.error(`Error simulating Solana transaction:`, simulateResponse.value.err);
                if (throwOnSimulateError) {
                    throw new Error(`Error simulating Solana transaction: ${simulateResponse.value.err.toString()}`);
                }
            }
            if ((_a = simulateResponse === null || simulateResponse === void 0 ? void 0 : simulateResponse.value) === null || _a === void 0 ? void 0 : _a.unitsConsumed) {
                // Set compute budget to 120% of the units used in the simulated transaction
                computeBudget = Math.round(simulateResponse.value.unitsConsumed * 1.2);
            }
        }
        catch (e) {
            console.error(`Failed to calculate compute unit limit for Solana transaction: ${e}`);
        }
        try {
            priorityFee = yield determinePriorityFee(connection, lockedWritableAccounts, feePercentile);
        }
        catch (e) {
            console.error(`Failed to calculate compute unit price for Solana transaction: ${e}`);
        }
        priorityFee = Math.max(priorityFee, minPriorityFee);
        return [
            ComputeBudgetProgram.setComputeUnitLimit({
                units: computeBudget,
            }),
            ComputeBudgetProgram.setComputeUnitPrice({
                microLamports: priorityFee,
            }),
        ];
    });
}
function determinePriorityFee(connection, lockedWritableAccounts = [], percentile) {
    return __awaiter(this, void 0, void 0, function* () {
        // https://twitter.com/0xMert_/status/1768669928825962706
        let fee = 1; // Set fee to 1 microlamport by default
        try {
            const recentFeesResponse = yield connection.getRecentPrioritizationFees({
                lockedWritableAccounts,
            });
            if (recentFeesResponse) {
                // Get 75th percentile fee paid in recent slots
                const recentFees = recentFeesResponse
                    .map((dp) => dp.prioritizationFee)
                    .filter((dp) => dp > 0)
                    .sort((a, b) => a - b);
                if (recentFees.length > 0) {
                    const medianFee = recentFees[Math.floor(recentFees.length * percentile)];
                    fee = Math.max(fee, medianFee);
                }
            }
        }
        catch (e) {
            console.error("Error fetching Solana recent fees", e);
        }
        return fee;
    });
}
